import time
from pubsub import PubSub

c = {}

# begin
p = PubSub(c)
publisher = p.pub()

# xpub = p.create_topic('test_001')
# print(xpub)
# xsub = p.create_subscription('test_001', 'test_sub_001')
# print(xsub)
3

pid = p.send(
    publisher, {
        'topic': 'test001',
        'namespace': 'auth',
        'subname': 'auth_register',
        'delay': 1,
        'data': {
            'fullname': 'mas joko',
            'email': 'panas@gmail.com'
        }
    })
print('send pubsub:', pid)


from bson import json_util as json
from google.auth import jwt
from google.cloud import pubsub_v1
from pydantic import BaseModel


class TaskData(BaseModel):
    topic: str
    namespace: str
    subname: str
    delay: int
    data: dict


class PubSub:
    def __init__(self, c):
        self.config = c

    def pub(self):
        audience = "https://pubsub.googleapis.com/google.pubsub.v1.Publisher"
        credentials = jwt.Credentials.from_service_account_info(
            self.config, audience=audience)
        publisher = pubsub_v1.PublisherClient(credentials=credentials)
        return publisher

    def sub(self):
        audience = "https://pubsub.googleapis.com/google.pubsub.v1.Subscriber"
        credentials = jwt.Credentials.from_service_account_info(
            self.config, audience=audience)
        subscriber = pubsub_v1.SubscriberClient(credentials=credentials)
        return subscriber

    def create_topic(self, name):
        px = self.pub()
        topic_path = px.topic_path(self.config["project_id"], name)
        try:
            px.create_topic(topic_path)
            return True
        except Exception:
            return False

    def create_subscription(self, topicname, name):
        ps = self.sub()
        subscription_path = ps.subscription_path(self.config["project_id"],
                                                 name)
        try:
            ps.create_subscription(subscription_path, topicname)
            return True
        except Exception:
            return False

    def send(self, publisher, data: TaskData) -> str:
        project_id = self.config['project_id']

        msg = json.dumps(data['data']).encode("utf-8")
        topic_path = publisher.topic_path(project_id, data.get('topic'))

        future = publisher.publish(topic_path,
                                   data=msg,
                                   delay=str(data.get('delay')),
                                   subname=data.get('subname'),
                                   namespace=data.get('namespace'))

        pubsub_id = future.result()
        return pubsub_id



import asyncio
from datetime import datetime
from pubsub import PubSub

c = {}


async def executor(data):
    print("{}".format(data.data), datetime.now())


async def sublistener(config, subname, count=1):
    '''Pubsub listen subscribtion tasks'''
    p = PubSub(c)
    while True:
        subscrb = p.sub()
        with subscrb:
            subpath = subscrb.subscription_path(config["project_id"], subname)

            # get messages
            response = subscrb.pull(request={
                'subscription': subpath,
                'max_messages': count
            })

            ack_ids = []
            for msg in response.received_messages:

                # print("Received: {}".format(msg.message))
                await executor(msg.message)

                ack_ids.append(msg.ack_id)

            # Acknowledges the received messages so they will not be sent again.
            tot = len(response.received_messages)
            if tot > 0:
                subscrb.acknowledge(request={
                    "subscription": subpath,
                    "ack_ids": ack_ids,
                })

        await asyncio.sleep(2)


# begin
async def main_pubsub(c):
    await sublistener(c, 'test001-sub', 10)


if __name__ == '__main__':

    loop = asyncio.new_event_loop()
    try:
        loop.create_task(main_pubsub(c))
        loop.run_forever()
    except KeyboardInterrupt:
        loop.close()
    finally:
        loop.close()
